SystemOrganization addCategory: #'Seaside-JSON-Core'!

!Collection methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:20'!
jsonOn: aRenderer 
	aRenderer array: [
		self do: [ :each |
			aRenderer value: each ] ]! !

WAObject subclass: #WAJsonParser
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonParser commentStamp: 'lr 12/31/2009 23:56' prior: 0!
I am a parser for JSON. It is a bit more forgiving than the standard and allows any kind of top level element except numbers, not just {} and []. See http://www.json.org/ for details.

I can be subclasses to create more sophisticated objects than just Arrays and Dictionaries. To do that, override the one or more methods in the creating protocol.!

WAJsonParser subclass: #JSJsonParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!JSJsonParser commentStamp: 'pmm 5/13/2012 10:12' prior: 0!
I'm purely for backwards compatiblity with Seaside 3.0!

!WAJsonParser class methodsFor: 'instance creation' stamp: 'lr 6/25/2009 15:58'!
new
	self error: 'Instantiate the parser with a stream.'! !

!WAJsonParser class methodsFor: 'instance creation' stamp: 'lr 6/25/2009 15:57'!
on: aStream
	^ self basicNew initializeOn: aStream! !

!WAJsonParser class methodsFor: 'accessing' stamp: 'lr 6/25/2009 15:55'!
parse: aString
	^ self parseStream: aString readStream! !

!WAJsonParser class methodsFor: 'accessing' stamp: 'lr 6/25/2009 15:57'!
parseStream: aStream
	^ (self on: aStream) parse! !

!WAJsonParser methodsFor: 'adding' stamp: 'lr 6/25/2009 20:06'!
addProperty: anAssociation to: anObject
	"Add the property anAssociation described with key and value to anObject. Subclasses might want to refine this implementation."
	
	^ anObject 
		add: anAssociation;
		yourself! !

!WAJsonParser methodsFor: 'adding' stamp: 'lr 9/22/2009 10:33'!
addValue: anObject to: aCollection
	"Add anObject to aCollection. Subclasses might want to refine this implementation."

	^ aCollection copyWith: anObject! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 6/25/2009 20:06'!
createArray
	"Create an empty collection. Subclasses might want to refine this implementation."

	^ Array new! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 10/13/2008 20:54'!
createFalse
	"Create the false literal. Subclasses might want to refine this implementation."
	
	^ false! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 10/13/2008 20:54'!
createNull
	"Create the null literal. Subclasses might want to refine this implementation."

	^ nil! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 10/13/2008 20:54'!
createNumber: aString
	"Create a number literal. Subclasses might want to refine this implementation."

	^ aString asNumber! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 6/25/2009 16:53'!
createObject
	"Create an empty object. Subclasses might want to refine this implementation."
	
	^ Dictionary new! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 10/13/2008 20:55'!
createProperty: aKey with: aValue
	"Create an empty attribute value pair. Subclasses might want to refine this implementation."
	
	^ aKey -> aValue! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 10/13/2008 20:54'!
createString: aString
	"Create a string literal. Subclasses might want to refine this implementation."

	^ aString! !

!WAJsonParser methodsFor: 'creating' stamp: 'lr 10/13/2008 20:55'!
createTrue
	"Create the true literal. Subclasses might want to refine this implementation."

	^ true! !

!WAJsonParser methodsFor: 'private' stamp: 'pmm 5/6/2012 16:23'!
error: aString
	"Raise a parse error labelled aString."

	^ WAJsonSyntaxError signal: aString! !

!WAJsonParser methodsFor: 'private' stamp: 'lr 6/29/2009 14:28'!
expect: aString
	"Expects aString and consume input, throw an error otherwise."

	^ (self match: aString) ifFalse: [ self error: aString , ' expected' ]! !

!WAJsonParser methodsFor: 'initialization' stamp: 'lr 1/25/2010 23:33'!
initializeOn: aStream
	self initialize.
	stream := aStream! !

!WAJsonParser methodsFor: 'private' stamp: 'pmm 5/1/2013 15:48'!
match: aString
	"Tries to match aString, consume input and answer true if successful."
	
	| position |
	self whitespace.
	position := stream position.
	aString do: [ :each |
		(stream atEnd or: [ stream next ~= each ]) ifTrue: [ 
			"reset position"
			stream position: position.
			^ false ] ].
	^ true! !

!WAJsonParser methodsFor: 'parsing' stamp: 'pmm 5/1/2013 15:48'!
parse
	| result |
	result := self whitespace; parseValue.
	self whitespace.
	stream atEnd
		ifFalse: [ self error: 'end of input expected' ].
	^ result! !

!WAJsonParser methodsFor: 'parsing' stamp: 'lr 9/8/2009 19:45'!
parseArray
	| result |
	self expect: '['.
	result := self createArray.
	(self match: ']')
		ifTrue: [ ^ result ].
	[ stream atEnd ] whileFalse: [
		result := self
			addValue: self parseValue
			to: result.
		(self match: ']') 
			ifTrue: [ ^ result ].
		self expect: ',' ].
	self error: 'end of array expected'! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 6/29/2009 14:39'!
parseCharacter
	| char |
	(char := stream next) = $\ 
		ifFalse: [ ^ char ].
	(char := stream next) = $" 
		ifTrue: [ ^ char ].
	char = $\
		ifTrue: [ ^ char ].
	char = $/
		ifTrue: [ ^ char ].
	char = $b
		ifTrue: [ ^ Character backspace ].
	char = $f
		ifTrue: [ ^ Character newPage ].
	char = $n
		ifTrue: [ ^ Character lf ].
	char = $r
		ifTrue: [ ^ Character cr ].
	char = $t
		ifTrue: [ ^ Character tab ].
	char = $u
		ifTrue: [ ^ self parseCharacterHex ].
	self error: 'invalid escape character \' , (String with: char)! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 7/16/2009 08:52'!
parseCharacterHex
	| value |
	value := self parseCharacterHexDigit.
	3 timesRepeat: [ value := (value << 4) + self parseCharacterHexDigit ].
	^ Character codePoint: value! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 7/26/2011 08:17'!
parseCharacterHexDigit
	| digit |
	stream atEnd ifFalse: [ 
		digit := stream next greaseInteger.
		(digit between: "$0" 48 and: "$9" 57)
			ifTrue: [ ^ digit - 48 ].
		(digit between: "$A" 65 and: "$F" 70)
			ifTrue: [ ^ digit - 55 ].
		(digit between: "$a" 97 and: "$f" 102)
			ifTrue: [ ^ digit - 87 ] ].
	self error: 'hex-digit expected'! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 6/25/2009 23:17'!
parseNumber
	| negated number |
	negated := stream peek = $-.
	negated ifTrue: [ stream next ].
	number := self parseNumberInteger.
	(stream peek = $.) ifTrue: [
		stream next. 
		number := number + self parseNumberFraction ].
	(stream peek = $e or: [ stream peek = $E ]) ifTrue: [
		stream next.
		number := number * self parseNumberExponent ].
	negated ifTrue: [ number := number negated ].
	^ self whitespace; createNumber: number! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 7/26/2011 08:18'!
parseNumberExponent
	| number negated |
	number := 0.
	negated := stream peek = $-.
	(negated or: [ stream peek = $+ ]) ifTrue: [ stream next ].
	[ stream atEnd not and: [ stream peek isDigit ] ]
		whileTrue: [ number := 10 * number + (stream next greaseInteger - 48) ].
	negated ifTrue: [ number := number negated ].
	^ 10 raisedTo: number! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 7/26/2011 08:18'!
parseNumberFraction
	| number power |
	number := 0.
	power := 1.0.
	[ stream atEnd not and: [ stream peek isDigit ] ] whileTrue: [
		number := 10 * number + (stream next greaseInteger - 48).
		power := power * 10.0 ].
	^ number / power! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 7/26/2011 08:18'!
parseNumberInteger
	| number |
	number := 0.
	[ stream atEnd not and: [ stream peek isDigit ] ]
		whileTrue: [ number := 10 * number + (stream next greaseInteger - 48) ].
	^ number! !

!WAJsonParser methodsFor: 'parsing' stamp: 'lr 9/8/2009 19:44'!
parseObject
	| result |
	self expect: '{'.
	result := self createObject.
	(self match: '}')
		ifTrue: [ ^ result ].
	[ stream atEnd ] whileFalse: [
		result := self
			addProperty: self parseProperty
			to: result.
		(self match: '}')
			ifTrue: [ ^ result ].
		self expect: ',' ].
	self error: 'end of object expected'! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'pmm 9/1/2012 16:28'!
parseProperty
	| name value |
	name := self parseString.
	self expect: ':'.
	value := self parseValue.
	^ self createProperty: name with: value! !

!WAJsonParser methodsFor: 'parsing-internal' stamp: 'lr 6/29/2009 14:40'!
parseString
	| result |
	self expect: '"'.
	result := WriteStream on: String new.
	[ stream atEnd or: [ stream peek = $" ] ] 
		whileFalse: [ result nextPut: self parseCharacter ].
	^ self expect: '"'; createString: result contents! !

!WAJsonParser methodsFor: 'parsing' stamp: 'pmm 5/1/2013 15:49'!
parseValue
	stream atEnd ifFalse: [ 
		| char |
		self whitespace.
		char := stream peek.
		char = ${
			ifTrue: [ ^ self parseObject ].
		char = $[
			ifTrue: [ ^ self parseArray ].
		char = $"
			ifTrue: [ ^ self parseString ].
		(char = $- or: [ char between: $0 and: $9 ])
			ifTrue: [ ^ self parseNumber ].
		(self match: 'true')
			ifTrue: [ ^ self createTrue ].
		(self match: 'false')
			ifTrue: [ ^ self createFalse ].
		(self match: 'null')
			ifTrue: [ ^ self createNull ] ].
	self error: 'invalid input'! !

!WAJsonParser methodsFor: 'private' stamp: 'lr 6/25/2009 23:01'!
whitespace
	"Strip whitespaces from the input stream."

	[ stream atEnd not and: [ stream peek isSeparator ] ]
		whileTrue: [ stream next ]! !

!Character methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 16:31'!
jsonOn: aRenderer
	aRenderer string: (String with: self)! !

!GRSmallDictionary methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:24'!
jsonOn: aRenderer
	aRenderer object: [
		self keysAndValuesDo: [ :key :value |
			aRenderer key: key value: value ] ]! !

WACanvas subclass: #WAJsonCanvas
	instanceVariableNames: 'first'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonCanvas class methodsFor: 'instance creation' stamp: 'pmm 2/18/2012 15:47'!
builder
	^ (WABuilder on: self)
		documentClass: WAJsonDocument;
		yourself! !

!WAJsonCanvas methodsFor: 'public' stamp: 'pmm 8/24/2011 13:57'!
array: aBlock
	^ (self brush: WAJsonArrayBrush new) with: aBlock! !

!WAJsonCanvas methodsFor: 'double-disptach' stamp: 'pmm 2/18/2012 14:47'!
boolean: aBoolean
	self document stream nextPutAll: (aBoolean ifTrue: [ 'true' ] ifFalse: [ 'false' ])! !

!WAJsonCanvas methodsFor: 'initialization' stamp: 'pmm 8/24/2011 15:05'!
initializeWithContext: aRenderContext
	super initializeWithContext: aRenderContext.
	first := true! !

!WAJsonCanvas methodsFor: 'public' stamp: 'pmm 8/24/2011 15:15'!
key: aKeyString value: aValueObject
	^ (self brush: (WAJsonKeyValueBrush key: aKeyString)) with: aValueObject! !

!WAJsonCanvas methodsFor: 'private' stamp: 'pmm 1/22/2012 16:28'!
nest: aBlock
	first := true.
	super nest: aBlock.
	first := false! !

!WAJsonCanvas methodsFor: 'double-disptach' stamp: 'pmm 2/18/2012 14:47'!
null
	self document stream nextPutAll: 'null'! !

!WAJsonCanvas methodsFor: 'double-disptach' stamp: 'pmm 2/18/2012 14:51'!
number: aNumber
	aNumber printOn: self document stream! !

!WAJsonCanvas methodsFor: 'public' stamp: 'pmm 8/24/2011 13:58'!
object: aBlock
	^ (self brush: WAJsonObjectBrush new) with: aBlock! !

!WAJsonCanvas methodsFor: 'public' stamp: 'pmm 2/18/2012 15:16'!
render: anObject
	self flush.
	anObject jsonOn: self! !

!WAJsonCanvas methodsFor: 'double-disptach' stamp: 'pmm 5/6/2012 15:30'!
string: aString
	self document string: aString! !

!WAJsonCanvas methodsFor: 'public' stamp: 'pmm 8/24/2011 14:25'!
value: aBlock
	^ (self brush: WAJsonValueBrush new) with: aBlock! !

!WAJsonCanvas methodsFor: 'private' stamp: 'pmm 8/24/2011 15:07'!
writeIfNotFirst: aString
	first ifFalse: [
		self document nextPutAll: aString ]! !

!Float methodsFor: '*seaside-json-core' stamp: 'pmm 6/24/2012 11:49'!
jsonOn: aRenderer
	(self - self) = 0.0 "check for infinity, negative infinity and NaN"
		ifTrue: [ aRenderer number: self ]
		ifFalse: [ aRenderer null ]! !

!BlockClosure methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:16'!
jsonOn: aRenderer
	self numArgs = 0
		ifTrue: [ self value ]
		ifFalse: [ self value: aRenderer ]! !

!Dictionary methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:23'!
jsonOn: aRenderer
	aRenderer object: [
		self keysAndValuesDo: [ :key :value |
			aRenderer key: key value: value ] ]! !

!GRPlatform methodsFor: '*seaside-json-core' stamp: 'pmm 5/6/2012 15:28'!
jsonEncoderOn: aStream
	"Answer an JSON encoder that works on aStream.
	Platforms may provide optimized implementations."
	^ WAJsonEncoder on: aStream! !

!GRPackage class methodsFor: '*seaside-json-core' stamp: 'pmm 5/6/2012 15:58'!
seasideJsonCore
	^ self new
		name: 'Seaside-JSON-Core';
		description: 'JSON integration in Seaside. Provides a basic functionality to generate JSON from Smalltalk.';
		addDependency: 'Seaside-Core';
		addDependency: 'Seaside-Canvas';
		url: #seasideUrl;
		yourself! !

!UndefinedObject methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:25'!
jsonOn: aRenderer
	aRenderer null! !

WAFileLibrary subclass: #WAJsonDeploymentLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonDeploymentLibrary methodsFor: 'uploads' stamp: 'lr 9/22/2009 10:31'!
json2Js
	^ 'if(!!this.JSON){this.JSON={};}(function(){function f(n){return n<10?"0"+n:n;}if(typeof Date.prototype.toJSON!!=="function"){Date.prototype.toJSON=function(key){return isFinite(this.valueOf())?this.getUTCFullYear()+"-"+f(this.getUTCMonth()+1)+"-"+f(this.getUTCDate())+"T"+f(this.getUTCHours())+":"+f(this.getUTCMinutes())+":"+f(this.getUTCSeconds())+"Z":null;};String.prototype.toJSON=Number.prototype.toJSON=Boolean.prototype.toJSON=function(key){return this.valueOf();};}var cx=/[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,escapable=/[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,gap,indent,meta={"\b":"\\b","\t":"\\t","\n":"\\n","\f":"\\f","\r":"\\r",''"'':''\\"'',"\\":"\\\\"},rep;function quote(string){escapable.lastIndex=0;return escapable.test(string)?''"''+string.replace(escapable,function(a){var c=meta[a];return typeof c==="string"?c:"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4);})+''"'':''"''+string+''"'';}function str(key,holder){var i,k,v,length,mind=gap,partial,value=holder[key];if(value&&typeof value==="object"&&typeof value.toJSON==="function"){value=value.toJSON(key);}if(typeof rep==="function"){value=rep.call(holder,key,value);}switch(typeof value){case"string":return quote(value);case"number":return isFinite(value)?String(value):"null";case"boolean":case"null":return String(value);case"object":if(!!value){return"null";}gap+=indent;partial=[];if(Object.prototype.toString.apply(value)==="[object Array]"){length=value.length;for(i=0;i<length;i+=1){partial[i]=str(i,value)||"null";}v=partial.length===0?"[]":gap?"[\n"+gap+partial.join(",\n"+gap)+"\n"+mind+"]":"["+partial.join(",")+"]";gap=mind;return v;}if(rep&&typeof rep==="object"){length=rep.length;for(i=0;i<length;i+=1){k=rep[i];if(typeof k==="string"){v=str(k,value);if(v){partial.push(quote(k)+(gap?": ":":")+v);}}}}else{for(k in value){if(Object.hasOwnProperty.call(value,k)){v=str(k,value);if(v){partial.push(quote(k)+(gap?": ":":")+v);}}}}v=partial.length===0?"{}":gap?"{\n"+gap+partial.join(",\n"+gap)+"\n"+mind+"}":"{"+partial.join(",")+"}";gap=mind;return v;}}if(typeof JSON.stringify!!=="function"){JSON.stringify=function(value,replacer,space){var i;gap="";indent="";if(typeof space==="number"){for(i=0;i<space;i+=1){indent+=" ";}}else{if(typeof space==="string"){indent=space;}}rep=replacer;if(replacer&&typeof replacer!!=="function"&&(typeof replacer!!=="object"||typeof replacer.length!!=="number")){throw new Error("JSON.stringify");}return str("",{"":value});};}if(typeof JSON.parse!!=="function"){JSON.parse=function(text,reviver){var j;function walk(holder,key){var k,v,value=holder[key];if(value&&typeof value==="object"){for(k in value){if(Object.hasOwnProperty.call(value,k)){v=walk(value,k);if(v!!==undefined){value[k]=v;}else{delete value[k];}}}}return reviver.call(holder,key,value);}cx.lastIndex=0;if(cx.test(text)){text=text.replace(cx,function(a){return"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4);});}if(/^[\],:{}\s]*$/.test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g,"@").replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g,"]").replace(/(?:^|:|,)(?:\s*\[)+/g,""))){j=eval("("+text+")");return typeof reviver==="function"?walk({"":j},""):j;}throw new SyntaxError("JSON.parse");};}}());'! !

!WAJsonDeploymentLibrary methodsFor: 'accessing' stamp: 'lr 9/22/2009 10:31'!
selectorsToInclude
	^ #(json2Js)! !

WAFileLibrary subclass: #WAJsonDevelopmentLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonDevelopmentLibrary methodsFor: 'uploads' stamp: 'lr 9/22/2009 10:29'!
json2Js
	^ '/*
    http://www.JSON.org/json2.js
    2009-08-17

    Public Domain.

    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

    See http://www.JSON.org/js.html

    This file creates a global JSON object containing two methods: stringify
    and parse.

        JSON.stringify(value, replacer, space)
            value       any JavaScript value, usually an object or array.

            replacer    an optional parameter that determines how object
                        values are stringified for objects. It can be a
                        function or an array of strings.

            space       an optional parameter that specifies the indentation
                        of nested structures. If it is omitted, the text will
                        be packed without extra whitespace. If it is a number,
                        it will specify the number of spaces to indent at each
                        level. If it is a string (such as ''\t'' or ''&nbsp;''),
                        it contains the characters used to indent at each level.

            This method produces a JSON text from a JavaScript value.

            When an object value is found, if the object contains a toJSON
            method, its toJSON method will be called and the result will be
            stringified. A toJSON method does not serialize: it returns the
            value represented by the name/value pair that should be serialized,
            or undefined if nothing should be serialized. The toJSON method
            will be passed the key associated with the value, and this will be
            bound to the value

            For example, this would serialize Dates as ISO strings.

                Date.prototype.toJSON = function (key) {
                    function f(n) {
                        // Format integers to have at least two digits.
                        return n < 10 ? ''0'' + n : n;
                    }

                    return this.getUTCFullYear()   + ''-'' +
                         f(this.getUTCMonth() + 1) + ''-'' +
                         f(this.getUTCDate())      + ''T'' +
                         f(this.getUTCHours())     + '':'' +
                         f(this.getUTCMinutes())   + '':'' +
                         f(this.getUTCSeconds())   + ''Z'';
                };

            You can provide an optional replacer method. It will be passed the
            key and value of each member, with this bound to the containing
            object. The value that is returned from your method will be
            serialized. If your method returns undefined, then the member will
            be excluded from the serialization.

            If the replacer parameter is an array of strings, then it will be
            used to select the members to be serialized. It filters the results
            such that only members with keys listed in the replacer array are
            stringified.

            Values that do not have JSON representations, such as undefined or
            functions, will not be serialized. Such values in objects will be
            dropped; in arrays they will be replaced with null. You can use
            a replacer function to replace those with JSON values.
            JSON.stringify(undefined) returns undefined.

            The optional space parameter produces a stringification of the
            value that is filled with line breaks and indentation to make it
            easier to read.

            If the space parameter is a non-empty string, then that string will
            be used for indentation. If the space parameter is a number, then
            the indentation will be that many spaces.

            Example:

            text = JSON.stringify([''e'', {pluribus: ''unum''}]);
            // text is ''["e",{"pluribus":"unum"}]''


            text = JSON.stringify([''e'', {pluribus: ''unum''}], null, ''\t'');
            // text is ''[\n\t"e",\n\t{\n\t\t"pluribus": "unum"\n\t}\n]''

            text = JSON.stringify([new Date()], function (key, value) {
                return this[key] instanceof Date ?
                    ''Date('' + this[key] + '')'' : value;
            });
            // text is ''["Date(---current time---)"]''


        JSON.parse(text, reviver)
            This method parses a JSON text to produce an object or array.
            It can throw a SyntaxError exception.

            The optional reviver parameter is a function that can filter and
            transform the results. It receives each of the keys and values,
            and its return value is used instead of the original value.
            If it returns what it received, then the structure is not modified.
            If it returns undefined then the member is deleted.

            Example:

            // Parse the text. Values that look like ISO date strings will
            // be converted to Date objects.

            myData = JSON.parse(text, function (key, value) {
                var a;
                if (typeof value === ''string'') {
                    a =
/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/.exec(value);
                    if (a) {
                        return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3], +a[4],
                            +a[5], +a[6]));
                    }
                }
                return value;
            });

            myData = JSON.parse(''["Date(09/09/2001)"]'', function (key, value) {
                var d;
                if (typeof value === ''string'' &&
                        value.slice(0, 5) === ''Date('' &&
                        value.slice(-1) === '')'') {
                    d = new Date(value.slice(5, -1));
                    if (d) {
                        return d;
                    }
                }
                return value;
            });


    This is a reference implementation. You are free to copy, modify, or
    redistribute.

    This code should be minified before deployment.
    See http://javascript.crockford.com/jsmin.html

    USE YOUR OWN COPY. IT IS EXTREMELY UNWISE TO LOAD CODE FROM SERVERS YOU DO
    NOT CONTROL.
*/

/*jslint evil: true */

/*members "", "\b", "\t", "\n", "\f", "\r", "\"", JSON, "\\", apply,
    call, charCodeAt, getUTCDate, getUTCFullYear, getUTCHours,
    getUTCMinutes, getUTCMonth, getUTCSeconds, hasOwnProperty, join,
    lastIndex, length, parse, prototype, push, replace, slice, stringify,
    test, toJSON, toString, valueOf
*/

"use strict";

// Create a JSON object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!!this.JSON) {
    this.JSON = {};
}

(function () {

    function f(n) {
        // Format integers to have at least two digits.
        return n < 10 ? ''0'' + n : n;
    }

    if (typeof Date.prototype.toJSON !!== ''function'') {

        Date.prototype.toJSON = function (key) {

            return isFinite(this.valueOf()) ?
                   this.getUTCFullYear()   + ''-'' +
                 f(this.getUTCMonth() + 1) + ''-'' +
                 f(this.getUTCDate())      + ''T'' +
                 f(this.getUTCHours())     + '':'' +
                 f(this.getUTCMinutes())   + '':'' +
                 f(this.getUTCSeconds())   + ''Z'' : null;
        };

        String.prototype.toJSON =
        Number.prototype.toJSON =
        Boolean.prototype.toJSON = function (key) {
            return this.valueOf();
        };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            ''\b'': ''\\b'',
            ''\t'': ''\\t'',
            ''\n'': ''\\n'',
            ''\f'': ''\\f'',
            ''\r'': ''\\r'',
            ''"'' : ''\\"'',
            ''\\'': ''\\\\''
        },
        rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        escapable.lastIndex = 0;
        return escapable.test(string) ?
            ''"'' + string.replace(escapable, function (a) {
                var c = meta[a];
                return typeof c === ''string'' ? c :
                    ''\\u'' + (''0000'' + a.charCodeAt(0).toString(16)).slice(-4);
            }) + ''"'' :
            ''"'' + string + ''"'';
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i,          // The loop counter.
            k,          // The member key.
            v,          // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (value && typeof value === ''object'' &&
                typeof value.toJSON === ''function'') {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === ''function'') {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value''s type.

        switch (typeof value) {
        case ''string'':
            return quote(value);

        case ''number'':

// JSON numbers must be finite. Encode non-finite numbers as null.

            return isFinite(value) ? String(value) : ''null'';

        case ''boolean'':
        case ''null'':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce ''null''. The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is ''object'', we might be dealing with an object or an array or
// null.

        case ''object'':

// Due to a specification blunder in ECMAScript, typeof null is ''object'',
// so watch out for that case.

            if (!!value) {
                return ''null'';
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// Is the value an array?

            if (Object.prototype.toString.apply(value) === ''[object Array]'') {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || ''null'';
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0 ? ''[]'' :
                    gap ? ''[\n'' + gap +
                            partial.join('',\n'' + gap) + ''\n'' +
                                mind + '']'' :
                          ''['' + partial.join('','') + '']'';
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === ''object'') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    k = rep[i];
                    if (typeof k === ''string'') {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? '': '' : '':'') + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? '': '' : '':'') + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0 ? ''{}'' :
                gap ? ''{\n'' + gap + partial.join('',\n'' + gap) + ''\n'' +
                        mind + ''}'' : ''{'' + partial.join('','') + ''}'';
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    if (typeof JSON.stringify !!== ''function'') {
        JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

            var i;
            gap = '''';
            indent = '''';

// If the space parameter is a number, make an indent string containing that
// many spaces.

            if (typeof space === ''number'') {
                for (i = 0; i < space; i += 1) {
                    indent += '' '';
                }

// If the space parameter is a string, it will be used as the indent string.

            } else if (typeof space === ''string'') {
                indent = space;
            }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

            rep = replacer;
            if (replacer && typeof replacer !!== ''function'' &&
                    (typeof replacer !!== ''object'' ||
                     typeof replacer.length !!== ''number'')) {
                throw new Error(''JSON.stringify'');
            }

// Make a fake root object containing our value under the key of ''''.
// Return the result of stringifying the value.

            return str('''', {'''': value});
        };
    }


// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !!== ''function'') {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === ''object'') {
                    for (k in value) {
                        if (Object.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !!== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return ''\\u'' +
                        (''0000'' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with ''()'' and ''new''
// because they can cause invocation, and ''='' because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE''s and Safari''s regexp engines. First we
// replace the JSON backslash pairs with ''@'' (a non-JSON character). Second, we
// replace all simple value tokens with '']'' characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or '']'' or
// '','' or '':'' or ''{'' or ''}''. If that is so, then the text is safe for eval.

            if (/^[\],:{}\s]*$/.
test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, ''@'').
replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, '']'').
replace(/(?:^|:|,)(?:\s*\[)+/g, ''''))) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The ''{'' operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval(''('' + text + '')'');

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return typeof reviver === ''function'' ?
                    walk({'''': j}, '''') : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError(''JSON.parse'');
        };
    }
}());'! !

!WAJsonDevelopmentLibrary methodsFor: 'accessing' stamp: 'lr 9/22/2009 10:31'!
selectorsToInclude
	^ #(json2Js)! !

!WAUrl methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:27'!
jsonOn: aRenderer
	aRenderer string: self printString! !

WATableBasedEncoder subclass: #WAJsonEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonEncoder class methodsFor: 'initialization' stamp: 'pmm 5/6/2012 15:21'!
initialize
	self initializeTable! !

!WAJsonEncoder class methodsFor: 'initialization' stamp: 'pmm 5/1/2013 14:53'!
initializeTable
	| printer escapes |
	printer := GRNumberPrinter new.
	printer base: 16; padding: $0; digits: 4.
	table := Array new: self maximumCharacterValue + 1.
	"write control characters"
	0 to: 31 do: [ :index | 
		table 
			at: index + 1
			put: '\u', (printer print: index) ].

	"overwrite named (control) characters"
	escapes := #( 8 '\b' 9 '\t' 10 '\n' 12 '\f' 13 '\r' 34 '\"' 92 '\\' ).
	1 to: escapes size - 1 by: 2 do: [ :index | 
		table 
			at: (escapes at: index) + 1
			put: (escapes at: index + 1) ]! !

!WAJsonEncoder class methodsFor: 'private' stamp: 'pmm 5/6/2012 15:20'!
maximumCharacterValue
	"Everything higher is safe."
	
	^ 16r7F! !

!WAJsonEncoder methodsFor: 'accessing' stamp: 'pmm 5/6/2012 15:27'!
nextPut: aCharacter 
	| index encoded |
	index := aCharacter greaseInteger + 1.
	index > table size ifTrue: [
		stream nextPut: aCharacter.
		^ self ].
	encoded := table at: index.
	"Issue 482: use #notNil because it is faster than #isString because it is not actually sent"
	encoded notNil
		ifTrue: [ stream nextPutAll: encoded ]
		ifFalse: [ stream nextPut: aCharacter ]! !

WAError subclass: #WAJsonSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonSyntaxError commentStamp: 'pmm 12/15/2008 19:45' prior: 0!
A JSJsonSyntaxError signals invalid JSON input.
!

!GRDelegatingStream methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:41'!
json: anObject
	| document context canvas |
	document := WAJsonDocument on: self.
	context := WARenderContext new
		actionUrl: WAUrl new;
		document: document;
		yourself.
	canvas := WAJsonCanvas context: context.
	canvas render: anObject.
	document close! !

!Integer methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:25'!
jsonOn: aRenderer
	aRenderer number: self! !

WADocument subclass: #WAJsonDocument
	instanceVariableNames: 'jsonEncoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonDocument methodsFor: 'writing' stamp: 'pmm 8/24/2011 12:08'!
closeArray
	stream nextPut: $]! !

!WAJsonDocument methodsFor: 'writing' stamp: 'pmm 8/24/2011 12:08'!
closeObject
	stream nextPut: $}! !

!WAJsonDocument methodsFor: 'initialization' stamp: 'pmm 9/1/2012 16:28'!
initializeWithStream: aStream codec: aCodec
	super initializeWithStream: aStream codec: aCodec.
	jsonEncoder := GRPlatform current jsonEncoderOn: aStream! !

!WAJsonDocument methodsFor: 'writing' stamp: 'pmm 8/24/2011 14:04'!
openArray
	stream nextPut: $[! !

!WAJsonDocument methodsFor: 'writing' stamp: 'pmm 8/24/2011 14:04'!
openObject
	stream nextPut: ${! !

!WAJsonDocument methodsFor: 'writing' stamp: 'pmm 5/6/2012 15:34'!
string: aString
	stream nextPut: $".
	jsonEncoder nextPutAll: aString.
	stream nextPut: $"! !

!Boolean methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:17'!
jsonOn: aRenderer
	aRenderer boolean: self! !

!Stream methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:42'!
json: anObject
	| document context canvas |
	document := WAJsonDocument on: self.
	context := WARenderContext new
		actionUrl: WAUrl new;
		document: document;
		yourself.
	canvas := WAJsonCanvas context: context.
	canvas render: anObject.
	document close! !

!Object methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:32'!
asJson
	^ WAJsonCanvas builder render: [ :json |
		self jsonOn: json ]! !

!Object methodsFor: '*seaside-json-core' stamp: 'jf 1/21/2013 06:56'!
jsonOn: aRenderer
	self error: 'Override #jsonOn: to make the receiver serializeable as JSON'! !

!String methodsFor: '*seaside-json-core' stamp: 'pmm 2/18/2012 15:21'!
jsonOn: aRenderer
	aRenderer string: self! !

WABrush subclass: #WAJsonBrush
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

WAJsonBrush subclass: #WAJsonArrayBrush
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonArrayBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 12:09'!
closeBrush
	self document closeArray! !

!WAJsonArrayBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 14:05'!
openBrush
	self document openArray! !

!WAJsonBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 12:08'!
closeBrush
	self subclassResponsibility! !

!WAJsonBrush methodsFor: 'accessing' stamp: 'pmm 8/24/2011 14:06'!
document
	^ canvas document! !

!WAJsonBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 14:00'!
openBrush
	self subclassResponsibility! !

!WAJsonBrush methodsFor: 'public' stamp: 'pmm 8/24/2011 14:07'!
with: anObject

	self openBrush.
	super with: [
		canvas render: anObject ].
	self closeBrush! !

WAJsonBrush subclass: #WAJsonElementBrush
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonElementBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 15:11'!
closeBrush! !

!WAJsonElementBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 15:11'!
openBrush
	canvas writeIfNotFirst: ', '! !

WAJsonElementBrush subclass: #WAJsonKeyValueBrush
	instanceVariableNames: 'key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonKeyValueBrush class methodsFor: 'instance creation' stamp: 'pmm 8/24/2011 15:10'!
key: aString
	^ self basicNew initializeWithKey: aString! !

!WAJsonKeyValueBrush methodsFor: 'initialization' stamp: 'pmm 8/24/2011 15:10'!
initializeWithKey: aString
	self initialize.
	key := aString! !

!WAJsonKeyValueBrush methodsFor: 'private' stamp: 'pmm 2/18/2012 15:16'!
openBrush
	super openBrush.
	key jsonOn: canvas.
	self document stream nextPutAll: ': '! !

WAJsonElementBrush subclass: #WAJsonValueBrush
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

WAJsonBrush subclass: #WAJsonObjectBrush
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-JSON-Core'!

!WAJsonObjectBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 12:10'!
closeBrush
	self document closeObject! !

!WAJsonObjectBrush methodsFor: 'private' stamp: 'pmm 8/24/2011 14:05'!
openBrush
	self document openObject! !
WAJsonEncoder initialize!
